import * as moment from './moment'
import * as rxjs from 'rx';

declare namespace gllue{

    export let Rx: typeof rxjs;

    export type ProcessTemplateContext = {
        context: any,
        type: any,
        noticeType: any
    }
    
    export interface ExtraNoticeProcessFunction { 
        processTemplate(template: string, templateCtx: ProcessTemplateContext): rxjs.Observable<string>;
    }
    export let addProcessFunction: (ExtraNoticeProcessFunction) => void;

    export let CompanyInfo:{
        CompanyName: string
        CompanyIndustry: string
        CompanyScale: string
        CompanyNatrue: string
    }

    export interface IEditModel {
        Name: string
        Xml?: IXml
        Data?: object
        Id?: number
        Scene?: 'detail' | 'list' | 'detailtab'
        AdHcoMap?: IAdHocMap
    }

    export class Edit{
        constructor(model:IEditModel);
        Title: string
        Icon: string
        BeforeSave:{():void};
        AfterSave:{():void};
        OnSaveSuccess:{():void};
        OnSaveFaild:{():void};
        GetData():any;
        Render():IJQueryObject;
    }

    export abstract class Modal{
        readonly Hide:{():void}
        Size: 'large' | 'large-plus' | 'full'
        readonly Show:{():void}
        abstract InitContent($ContentContainer:IJQueryObject)
    }

    export class RestfulApi {
        constructor(Modelname:string)
        GteUrlByPath(path:string):string
        static GteExtensionApiUrl(path:string):string
    }

    export class DataFetcher {
        constructor(ModelName: string)
        List(filter: string): Promise<any[]>
        Detail(id: number, fields?: string[]): Promise<any>
        Add(param: any, id?: number): Promise<any>
        Delete(id: number): Promise<any>
    }

    export type IAdHocMap = {[key:string]: IVue}

    export interface IModelAdhocs {
        Model: string,
        Scene?: string,
        Adhocs: IAdHocMap
    }

    export abstract class Composer {
        Size: string
        readonly Show:{():void}
        abstract Title: string
        abstract InitContent($ContentContainer:IJQueryObject)
    }

    export interface IVue {

    }

    export class Vue {
        constructor(vueoptions:any)
        static component(name: string, opt:any):any
    }

    export interface IXml {

    }

    export interface IAjaxParam{
        Data?: object
        Url: string
        Type?: 'get' | 'post'
    }

    export class Document{
        static Ajax<T>(param:IAjaxParam):Promise<T>
        static Ajax(param:IAjaxParam):Promise<any>
        static Listen(event:string, func:{(args:any):void}):void
    }

    export class Window{
        static OpenInNewTab(url:string):void
        static ShowModal(modal:Modal):void
        static ShowModal(type:string, name: string, param:any):void
        static Message(type: 'success' | 'error', content: string):void;
    }

    export interface IDetailDescription{
        Id: number
        Json?: any
    }

    export type Order = number | 'first' | 'last'

    export interface IDetailTabItem{
        readonly DetailDes:IDetailDescription
        Order:  Order
        Icon:string
        Title:string
        Render():Promise<IJQueryObject>
        AfterRender($e:IJQueryObject):void
    }

    export interface IDetailTabItemConstructor{
        Name:string
        ModelName:string
        new(DetailDes:IDetailDescription):IDetailTabItem
    }

    export interface ICustomActionProps{
        id: string
    }

    export interface ICustomActionItem{
        Model: string
        Scene?:  string[]
        Name:string
        Icon: string
        Label: string
        Place?: string
        ActionType: string
        HandlerClick(ICustomActionProps):any
    }

    export interface IModelCustomActions{
        Model: string
        CustomActions: ICustomActionItem[]
    }

    export interface FkSelectProps{
        Model: string
        Value: string
        ContainerId: string
        OnChange:{(val:TreeNode):void}
    }

    export interface ISpecialModel{
        [key:string]:  string[]
    }

    export interface ISpecialModelWithDependence{
        SpecialModels: ISpecialModel,
        Dependence?: string[]
    }

    export interface IMenuItem{
        path: string
        label: string
    }

    export interface IRouter{
        path: string
        handlerRouterchange(any): any
    }


    export type IJQueryObject = any

    export let JQuery:any;

    export let ModalBaseComponent: any;

    export function RegisterDetailTabItem(Item:IDetailTabItemConstructor):any;

    export function RegisterCustomActionItem(Item:ICustomActionItem):any;

    export function RegisterCustomPageItem(Item:IMenuItem, IRouters: IRouter[]):any;

    export function RegisterAdHocMap(Item:IModelAdhocs):any;

    export function RegisterCustomAction(Item:IModelCustomActions):any;

    export function RegisterSpecialModel(Item:ISpecialModelWithDependence):any;

    export function RegisterFeed(Item: IFeed):any

    export function RegisterCustomPageItem(Item:IMenuItem, IRouters: IRouter[], IAdhocs?: IModelAdhocs, ICustomActions?: IModelCustomActions, ISpecialModelWithDependence?: ISpecialModelWithDependence):any;

    export function GetTrans(text:string):string;

    export function GetTransByNamespace(text:string, namespace:string):string;

    export let Moment: typeof moment

    export function QueryStringToHash(querystring:string):any;

    export let Chart:{
        Echarts:any
    }

    export interface TreeNode{
        Label?: string
        Id: number
    }

    export interface TreeComponentProps{
        ContainerId: string
        OnChange:{(val:TreeNode):void}
        Style?:{[key:string]:string}
        Value?: number[]
        Class?:string
        Muti?: boolean
        Type?: 'City' | 'Function' | 'Industry' | 'Team'
        Defaultfirst?: boolean
        Url?: string
    }

    export interface SelectComponentProps{
        ContainerId: string
        OnChange:{(val:number):void}
        Model: string
        Value?: number
        Style?:{[key:string]:string}
        Class?:string
    }

    export interface EditWithXmlComponentProps{
        Model: string
        EditFormLayout: string
        Scene?: string
        Id?: string
    }

    export interface DetailWithXmlComponentProps{
        Model: string
        DetailLayout: string
        DetailFormLayout: string
        Id: string
    }

    export interface EditMainComponentProps{
        Model: string
        Scene?: string
        Id?: string
    }

    export interface DetailMainComponentProps{
        Model: string
        Id: string
    }

    export interface ListMainComponentProps{
        Model: string,
        HasSidebar?: boolean
    }

    export interface FkSelectComponentProps {
        Model: string,
        Filter: string,
        Url?: string,
        GetSelfData?(Url: string): Promise<any>
        MutiSelect: boolean,
        Onchange?(ids: string[]): void
    }

    export class Component{
        static Tree(props:TreeComponentProps):Promise<void>;
        static Select(props:SelectComponentProps):Promise<void>;
        static FkSelect(props:FkSelectComponentProps):any;
        static FkSelect(props:FkSelectProps):any;
        static EditWithXml(props:EditWithXmlComponentProps):string;
        static DetailWithXml(props:DetailWithXmlComponentProps):string;
        static EditMain(props:EditMainComponentProps):string;
        static DetailMain(props:DetailMainComponentProps):string;
        static List(props:ListMainComponentProps):string;
    }

    export class Modelbasecomponent{
        static BaseCompnpent: any;
        static BaseComponent: any
        static ModelBaseEdit: any
        static ModelBaseDetail: any
        static ModelBaseList: any
        static ModelBaseCard: any
        static ModelBaseKanbanitem: any
    }

    export class ModelbaseManager{
        static EditManager: any
        static DetailManager: any
        static CardManager: any
        static ListCardManager: any
        static ListManager: any
    }

    export class Router{
        static ListRenderRouter: any
    }

    export class DataHelper{
        static List: any
        static ModelMain: any
    }

    export class CompanentHelper{
        static ComponentManager: any
    }

    export let MetaFieldBase:any
    export let TypeRegister:any

    export let Lang: 'zh_CN' | 'en' | 'zh_TW'

    export let Uuid: (numeric:boolean) => string

    export interface ISnippt{
        label: string;
        dep : string[];
        category: { label: string; name: string; };
        code : string;
    }

    export function RegSnippet(newSnippt : ISnippt) :void

    export interface IFeed {
        Type: string,
        ComponentName: string,
        RequireFields?: string[],
    }

    export let CandidateStatusBarForReg:any

    export function RComponent(vueoptions:any): any;
    export function Watch(name: string, options?: any):any;
    export function Prop(options): any;

    export interface IDispose{
        Dispose():void
    }

    type EmailActionHandler = {
        new(noticeMgr:any): {
            GetComponentName(): string
        } & IDispose
    };
    
    export function RegEmailEditActions(handler:EmailActionHandler):void

}

export = gllue;
